import _thread
import network
from esp import espnow
import time

from machine import Pin, I2C
from ssd1306 import SSD1306_I2C    
i2c = I2C(scl=Pin(22),sda=Pin(21),freq=100000)
oled = SSD1306_I2C(128,64,i2c)

BROADCAST = b'\xFF' * 6

# locker states
states = {
    "ready": 0, # unit -> center
    "startBooking": 1, # center -> unit
    "booked": 2, # unit -> center (ACK of startBooking)
    "used": 3 # unit -> center
}
stateNames = {
    0: "Available",
    1: "Booked",
    2: "Booked",
    3: "In Use"
}
lockers = [
    { "mac": b'\xa4\xcf\x12\x8f\xcc\x84', "state": states["ready"] },
    { "mac": b'\xa4\xcf\x12\x8f\xb8D', "state": states["ready"] },
    { "mac": b'$o(P\xb10', "state": states["ready"] }
]
isNotBooking = True
def display_lockers():
    global isNotBooking

    print("Thread start!", isNotBooking)
    while isNotBooking:
        oled.fill(0)
        for i in range(len(lockers)):
            oled.text("#{}: {}".format(i+1,stateNames[lockers[i]["state"]]),0,i*16)
        if isNotBooking: oled.show()
        else: break
    print("Thread broken!")
    
def receive_callback(*dobj):
    global lockers
    global isNotBooking
    mac, msg = dobj[0]

    msg = str(msg, 'utf8')
    for i in range(len(lockers)):
        if lockers[i]["mac"] == mac: lockers[i]["state"] = states[msg]
        
def espnowThread():
    global lockers
    w = network.WLAN()
    w.active(True)
    
    espnow.init()
    espnow.add_peer(BROADCAST)
    for locker in lockers:
        espnow.add_peer(locker["mac"])
    espnow.on_recv(receive_callback)

pins = [14,12,26,27] # 4x1 module pin : 4, 5, 2, 3 (1 is GND)
k_in = [Pin(i, Pin.IN, Pin.PULL_UP) for i in pins]

def bookLocker(no):
    global isNotBooking
    
    no -= 1
    if lockers[no]["state"] != states["ready"]:
        return False
    
    isNotBooking = False
    time.sleep(0.005)
    try:
        espnow.send(lockers[no]["mac"], "startBooking")
    except:
        pass
    lockers[no]["state"] = states["startBooking"]
    
    oled.fill(0)
    oled.text("Booking #{}...".format(no+1),0,24)
    oled.show()
    
    count = 5
    while count > 0 and lockers[no]["state"] != states["booked"] and lockers[no]["state"] == states["startBooking"]:
        count -= 1
        time.sleep(1)
        continue
    
    if lockers[no]["state"] == states["booked"]:
        oled.fill(0)
        oled.text("Booking Success!",0,16)
        oled.text("Cont. at locker",0,32)
        oled.show()
        time.sleep(3)
        
        isNotBooking = True
        _thread.start_new_thread(display_lockers, ())
        return True
    
    oled.fill(0)
    oled.text("Booking Error!",0,16)
    oled.text("Try again later",0,32)
    oled.show()
    time.sleep(3)
    lockers[no]["state"] = states["ready"]
    
    isNotBooking = True
    _thread.start_new_thread(display_lockers, ())
    return False

def digit():
    while True:
        for i in range(len(k_in)):
            if k_in[i].value() == 0:
                return i + 1
            time.sleep(0.000001) # 1us

_thread.start_new_thread(espnowThread, ())
_thread.start_new_thread(display_lockers, ())

while True:
    n = digit()
    while n not in [1, 2, 3]:
        n = digit()
        time.sleep(0.000125) # 0.125ms
    bookLocker(n)

    