import _thread
from machine import PWM, Pin, I2C
from ssd1306 import SSD1306_I2C
import time
import network
from esp import espnow

i2c = I2C(scl=Pin(22),sda=Pin(21),freq=100000)
ledGreen = Pin(19, Pin.OUT)
ledRed = Pin(18, Pin.OUT)
ledVCC = Pin(5, Pin.OUT)
ledVCC.value(1)
oled = SSD1306_I2C(128,64,i2c)
def servoOpen():
    print("servoOpen")
    servo = PWM(Pin(4), freq=50)
    for i in reversed(range(30, 130)):
        servo.duty(i)
        time.sleep(0.025)
def servoClose():
    print("servoClose")
    servo = PWM(Pin(4), freq=50)
    for i in range(130):
        servo.duty(i)
        time.sleep(0.025)
def buzzerBeep():
    print("buzzerBeep")
    for i in range(1):
        time.sleep(0.1)
        buzzer = PWM(Pin(15), freq=1000, duty=50)
        time.sleep(0.5)
        buzzer.deinit()
def doorBuzzerBeep():
    print("doorBuzzerBeep")
    for i in range(3):
        time.sleep(0.5)
        buzzer = PWM(Pin(15), freq=1000, duty=50)
        time.sleep(0.5)
        buzzer.deinit()
    
def doorOpen():
    _thread.start_new_thread(servoOpen, ())
def doorClose():
    _thread.start_new_thread(servoClose, ())
def keyBuzzer():
    _thread.start_new_thread(buzzerBeep, ())    
def doorBuzzer():
    _thread.start_new_thread(doorBuzzerBeep, ())
def led_ready():
    ledGreen.value(1)
    ledRed.value(0)
def led_used():
    ledGreen.value(0)
    ledRed.value(1)

BF_SIZE = 16
class keypad():
    def __init__(self, colPins = [], rowPins = []):
        self.State = {
            "Off_state": 0,
            "Off_to_on_transient": 1,
            "On_state": 2,
            "On_to_off_transient": 3
        }
        self.CHNG_CNT = 3
        
        self.k_in = [Pin(i, Pin.IN, Pin.PULL_UP) for i in rowPins]
        self.k_out = [Pin(j, Pin.OUT) for j in colPins]
        
        for jj in range(len(self.k_out)): self.k_out[jj].value(1)
        self.key_state = []
        self.key_transent_cntr = []
        
        for j in range(len(self.k_out)):
            key_state = []
            key_transent_cntr = []
            for i in range(len(self.k_in)):
                key_state.append(self.State["Off_state"])
                key_transent_cntr.append(self.CHNG_CNT)
            self.key_state.append(key_state)
            self.key_transent_cntr.append(key_transent_cntr)
        
        self.buf = []
        self.read_addr = self.write_addr = 0
        for i in range(BF_SIZE): self.buf.append(None)
        _thread.start_new_thread(self.ticker, ())
    
    def ticker(self):
        while True:
            self.key_scan()
            time.sleep(0.002) # 2ms

    def key_scan(self):
        for j in range(len(self.k_out)):
            for jj in range(len(self.k_out)): self.k_out[jj].value(1)
            self.k_out[j].value(0)
            time.sleep(0.000001) # 1us
            for i in range(len(self.k_in)):
                if self.key_state[j][i] == self.State["Off_state"]:
                    if self.k_in[i].value() == 0:
                        self.key_state[j][i] = self.State["Off_to_on_transient"]
                        self.key_transent_cntr[j][i] = self.CHNG_CNT
                elif self.key_state[j][i] == self.State["Off_to_on_transient"]:
                    if self.k_in[i].value() == 0:
                        self.key_transent_cntr[j][i] -= 1
                        if self.key_transent_cntr[j][i] < -self.CHNG_CNT:
                            print(i, j)
                            self.bf_put(j * len(self.k_in) + i + 1)
                            self.key_state[j][i] = self.State["On_state"]
                    else:
                        self.key_transent_cntr[j][i] += 1
                        if self.key_transent_cntr[j][i] > self.CHNG_CNT:
                            self.key_state[j][i] = self.State["Off_state"]
                elif self.key_state[j][i] == self.State["On_state"]:
                    if self.k_in[i].value() == 1:
                        self.key_state[j][i] = self.State["On_to_off_transient"]
                        self.key_transent_cntr[j][i] = -self.CHNG_CNT
                elif self.key_state[j][i] == self.State["On_to_off_transient"]:
                    if self.k_in[i].value() == 0:
                        self.key_transent_cntr[j][i] -= 1
                        if self.key_transent_cntr[j][i] < -self.CHNG_CNT:
                            self.key_state[j][i] = self.State["On_state"]
                    else:
                        self.key_transent_cntr[j][i] += 1
                        if self.key_transent_cntr[j][i] > self.CHNG_CNT:
                            self.key_state[j][i] = self.State["Off_state"]
                else:
                    self.key_state[j][i] = self.State["Off_state"]
        for jj in range(len(self.k_out)): self.k_out[jj].value(1)
    def read(self):
        return self.bf_get()
    def bf_put(self, dt):
        next_addr = self.write_addr + 1
        if next_addr == BF_SIZE: next_addr = 0
        self.buf[self.write_addr] = dt
        self.write_addr = next_addr
    def bf_get(self):
        if self.read_addr == self.write_addr: return None
        dt = self.buf[self.read_addr]
        self.read_addr += 1
        if self.read_addr == BF_SIZE: self.read_addr = 0
        return dt


kp = keypad([25,33,32], [12,14,27,26])
key_table = "?147*2580369#"
def digit():
    # Loop while waiting for a keypress
    digitPressed = None
    while digitPressed == None:
        digitPressed = kp.read()
    return key_table[digitPressed]
def passwordPrompt(firstLine):
    oled.fill(0)
    oled.text(firstLine,0,0)
    oled.text("your password",0,16)
    oled.show()
    password = []
    for i in range(4):
        password.append(digit())
        keyBuzzer()
        oled.text("*",i*8,32)
        oled.show()
    return "".join(password)

def mainFunc():
    status = True
    while True:
        password = passwordPrompt("Enter")
        if password == passwordSet:
            oled.fill(0)
            oled.text("DOOR UNLOCKED!",0,0)
            oled.text("Press # to lock",0,16)
            oled.text("Press * to end ",0,32)
            oled.show()
            time.sleep(0.5)
            doorOpen()        
            time.sleep(5)
            ##mainFunc()
            while True :
                confirm = digit()
                if confirm == '#':  ### edit by ohm
                    doorClose()
                    status = True
                    break
                elif confirm == '*' :
                    doorClose()
                    status = False
                    break      
        else:
            oled.fill(0)
            oled.text("Pass Incorrect!",0,0)
            oled.text("Try in 3 sec.",0,16)
            oled.show()
            time.sleep(3)
        if status == False:
            break
            

############################################################################### State send-recieve 
def Connection_State_1():
    count = 1
    print('State_1')
    while True:
        msg = "ready"
        led_ready()
        print("Sending:", msg)
        print(count)
        espnow.send(Center, msg) ## fill mac's center
        count += 1
        time.sleep(1)
        if Confirm_state == True :
            break
    
def Connection_State_2():
    msg = "booked"
    led_used()
    print('State_2')
    print("Sending:", msg)
    espnow.send(Center, msg) ## fill mac's center
    time.sleep(1)
    while True:
        passwordSet = passwordPrompt("Create")
        if passwordSet == passwordPrompt("CONFIRM"): return passwordSet
        oled.fill(0)
        oled.text("PWD mismatch!",0,16)
        oled.text("Try in next 3s.",0,32)
        oled.show()
        time.sleep(3)
    
def Connection_State_3():
    print('State_3')
    oled.fill(0)
    oled.text("WELCOME!",0,0)
    oled.text("Press # to lock.",0,16)
    oled.show()
    msg = "used"
    led_used()
    print("Sending:", msg)
    espnow.send(Center, msg)
    time.sleep(0.5)
    doorOpen() 
    while True:
        confirm = digit()
        if confirm == '#':
            doorClose()
            break
       
    
############################################################################### 

############################################################################### Send-Recieve
def receive_callback(*dobj):
    ##print(dobj)
    mac, msg = dobj[0]
    msg = str(msg,'utf8')
    if msg == 'startBooking':
        global Confirm_state
        Confirm_state = True

w = network.WLAN()
w.active(True)
Center = b'\xa4\xcf\x12\x83\xb9X'

espnow.init()
espnow.add_peer(Center) 
espnow.on_recv(receive_callback)
###############################################################################

Confirm_state = False
passwordSet = '' ## get from password_Create

import network; print(network.WLAN().config('mac'))

## Main State ##
while(1):
    passwordSet = 'qwer'
    Connection_State_1()
    ##print(55)
    passwordSet = Connection_State_2()
    Connection_State_3()
    mainFunc()
    Confirm_state = False
    oled.fill(0)
    oled.show()
################






